var EventUtil = {
	addHandler: function(element, type, handler) {
		if (element.addEventListener)
			element.addEventListener(type, handler, false);
		else if (element.attachEvent)
			element.attachEvent("on" + type, handler);
		else
			element["on" + type] = handler;
	},
	removeHandler: function(element, type, handler) {
		if (element.removeEventListener)
			element.removeEventListener(type, handler, false);
		else if (element.detachEvent)
			element.detachEvent("on" + type, handler);
		else
			element["on" + type] = handler;
	},
	/**
	 * 监听触摸的方向
	 * @param target            要绑定监听的目标元素
	 * @param isPreventDefault  是否屏蔽掉触摸滑动的默认行为（例如页面的上下滚动，缩放等）
	 * @param upCallback        向上滑动的监听回调（若不关心，可以不传，或传false）
	 * @param rightCallback     向右滑动的监听回调（若不关心，可以不传，或传false）
	 * @param downCallback      向下滑动的监听回调（若不关心，可以不传，或传false）
	 * @param leftCallback      向左滑动的监听回调（若不关心，可以不传，或传false）
	 */
	listenTouchDirection: function(target, isPreventDefault, upCallback, rightCallback, downCallback, leftCallback) {
		this.addHandler(target, "touchstart", handleTouchEvent);
		this.addHandler(target, "touchend", handleTouchEvent);
		this.addHandler(target, "touchmove", handleTouchEvent);
		var startX;
		var startY;

		function handleTouchEvent(event) {
			switch (event.type) {
				case "touchstart":
					startX = event.touches[0].pageX;
					startY = event.touches[0].pageY;
					break;
				case "touchend":
					var spanX = event.changedTouches[0].pageX - startX;
					var spanY = event.changedTouches[0].pageY - startY;

					if (Math.abs(spanX) > Math.abs(spanY)) { //认定为水平方向滑动
						if (spanX > 30) { //向右
							if (rightCallback)
								rightCallback(spanX);
						} else if (spanX < -30) { //向左
							if (leftCallback)
								leftCallback(spanX);
						}
					} else { //认定为垂直方向滑动
						if (spanY > 30) { //向下
							if (downCallback)
								downCallback(spanY);
						} else if (spanY < -30) { //向上
							if (upCallback)
								upCallback(spanY);
						}
					}

					break;
				case "touchmove":
					//阻止默认行为
					if (isPreventDefault)
						event.preventDefault();
					break;
			}
		}
	}
};

var $ = function(id) {
	return "string" == typeof id ? document.getElementById(id) : id;
};
var Class = {
	create: function() {
		return function() {
			this.initialize.apply(this, arguments);
		}
	}
}
Object.extend = function(destination, source) {
	for (var property in source) {
		destination[property] = source[property];
	}
	return destination;
}
var Calendar = Class.create();
Calendar.prototype = {
	initialize: function(container, options) {
		this.Container = $(container); //table结构容器
		this.Days = []; //日期列表 
		this.SetOptions(options);
		this.Year = this.options.Year;
		this.Month = this.options.Month;
		this.SelectDay = this.options.SelectDay ? new Date(this.options.SelectDay) : null;
		this.onSelectDay = this.options.onSelectDay;
		this.onToday = this.options.onToday;
		this.onFinish = this.options.onFinish;
		this.Draw();
	},

	SetOptions: function(options) {
		this.options = { //默认值 
			Year: new Date().getFullYear(),
			Month: new Date().getMonth() + 1,
			SelectDay: null, //选择日期 
			onSelectDay: function() {},
			onToday: function() {},
			onFinish: function() {}
		};
		Object.extend(this.options, options || {});
	},
	//上月 
	PreMonth: function() {
		//取得上月日期对象 
		var d = new Date(this.Year, this.Month - 2, 1);
		//设置属性 
		this.Year = d.getFullYear();
		this.Month = d.getMonth() + 1;
		//重绘日历 
		this.Draw();


	},
	//下一个月 
	NextMonth: function() {
		var d = new Date(this.Year, this.Month, 1);
		this.Year = d.getFullYear();
		this.Month = d.getMonth() + 1;
		this.Draw();

	},

	Draw: function() {
		//保存日期列表 
		var arr = [];
		var arr1 = [];
		var arr2 = [];
		//用当月第一天在一周中的日期值作为当月离第一天的天数 
		var firstDay = new Date(this.Year, this.Month - 1, 1).getDay();
		var upperGetDate = new Date(this.Year, this.Month - 1, 0).getDate(); //上个月天数
		var WhenGetDate = new Date(this.Year, this.Month, 0).getDate() //当月天数
		for (var i = firstDay - 1; i >= 0; i--) {
			arr.push(' ');
			arr1.push(upperGetDate - i);
		}
		//用当月最后一天在一个月中的日期值作为当月的天数 
		for (var i = 1, monthDay = new Date(this.Year, this.Month, 0).getDate(); i <= monthDay; i++) {
			arr.push(i);
		}
		var Make = arr.length % 7
		if ((Make % 7 == 0) ? false : true) {
			var s = 7 - Make
			for (var i = 1; i <= s; i++) {
				arr2.push(i);
			}
		}
		// /
		var frag = document.createDocumentFragment();
		this.Days = [];
		while (arr.length > 0) {
			//每个星期插入一个tr
			var row = document.createElement("tr");
			//星期 
			for (var i = 1; i <= 7; i++) {
				var cell = document.createElement("td");
				cell.innerHTML = " ";
				if (arr.length > 0) {
					var d = arr.shift();
					cell.innerHTML =  '<i>'+d+ '</i>';
					if (d > 0) {
						//获取今日 
						if (this.IsSame(new Date(this.Year, this.Month - 1, d), new Date())) {
							this.onToday(cell);
						}
						//判断用户是否作了选择
						if (this.SelectDay && this.IsSame(new Date(this.Year, this.Month - 1, d), this.SelectDay)) {
							this.onSelectDay(cell);
						}
					}
					this.Days[d] = cell;
				}
				row.appendChild(cell);
			}
			frag.appendChild(row);
		}




		//此先清空然后再插入(ie的table不能用innerHTML) 
		while (this.Container.hasChildNodes()) {
			this.Container.removeChild(this.Container.firstChild);
		}
		this.Container.appendChild(frag);
		this.onFinish();
		if (arr1.length > 0) {
			for (var i = 0; i < arr1.length; i++) {
				var table = document.getElementById("idCalendar"); //获取第一个表格
				var child = table.getElementsByTagName("td")[i]; //获取行的第一个单元格
				child.innerHTML = '<i style="color: #D8DDE6;">' + arr1[i] + '</i>'
			}
		}

		if (arr2.length > 0) {
			var s = WhenGetDate + arr1.length
			for (var i = 0; i < arr2.length; i++) {
				var table = document.getElementById("idCalendar"); //获取第一个表格
				var child = table.getElementsByTagName("td")[s + i]; //获取行的第一个单元格
				child.innerHTML = '<i style="color: #D8DDE6;">' + arr2[i] + '</i>'
			}
		}
	},
	//判断是否同一日
	IsSame: function(d1, d2) {
		return (d1.getFullYear() == d2.getFullYear() && d1.getMonth() == d2.getMonth() && d1.getDate() == d2.getDate());
	}
};
