/*******************************************************************************
 * Copyright (c) 2015 IBM Corp.
 *
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * and Eclipse Distribution License v1.0 which accompany this distribution.
 *
 * The Eclipse Public License is available at
 *    http://www.eclipse.org/legal/epl-v10.html
 * and the Eclipse Distribution License is available at
 *   http://www.eclipse.org/org/documents/edl-v10.php.
 *
 * Contributors:
 *    James Sutton - Initial Contribution
 *******************************************************************************/

/*
Eclipse Paho MQTT-JS Utility
This utility can be used to test the Eclipse Paho MQTT Javascript client.
*/

// Create a client instance
var client = null;
var connected = false;

function mqttNew(did) {
	connect()
	function connect() {
		var hostname = baseWS;
		if (Storage.getItem('isOfficial') == 'true') {
			var port = 443; //正式443  测试80
			var tls = true; //  true是加密   false没加密
		} else {
			var port = 80; //正式443  测试80
			var tls = false; //  true是加密   false没加密
		}
		var clientId = "js-utility-" + makeid();
		var path = '/mqtt';
		var user = 'admin';
		var pass = '111111';
		var keepAlive = 60;
		var timeout = 3;
		var automaticReconnect = true;
		var cleanSession = true;

		var lastWillTopic = '';
		var lastWillQos = '';
		var lastWillRetain = true;
		var lastWillMessageVal = '';


		if (path.length > 0) {
			client = new Paho.Client(hostname, Number(port), path, clientId);
		} else {
			client = new Paho.Client(hostname, Number(port), clientId);
		}
		logMessage("INFO", "正在连接到服务器: [Host: ", hostname, ", Port: ", port, ", Path: ", client.path, ", ID: ", clientId, "]");

		// 设置回调处理程序
		client.onConnectionLost = onConnectionLost;
		client.onMessageArrived = onMessageArrived;
		client.onConnected = onConnected;


		var options = {
			invocationContext: {
				host: hostname,
				port: port,
				path: client.path,
				clientId: clientId
			},
			timeout: timeout,
			keepAliveInterval: keepAlive,
			cleanSession: cleanSession,
			useSSL: tls,
			reconnect: automaticReconnect,
			onSuccess: onConnect,
			onFailure: onFail
		};

		if (user.length > 0) {
			options.userName = user;
		}

		if (pass.length > 0) {
			options.password = pass;
		}

		if (lastWillTopic.length > 0) {
			var lastWillMessage = new Paho.Message(lastWillMessageVal);
			lastWillMessage.destinationName = lastWillTopic;
			lastWillMessage.qos = lastWillQos;
			lastWillMessage.retained = lastWillRetain;
			options.willMessage = lastWillMessage;
		}

		// 连接客户端
		client.connect(options);
		console.log('连接...')

	}

	function disconnect() {
		logMessage("INFO", "断开与服务器的连接.");
		client.disconnect();
		console.log('连接 - 断开.')
		connected = false;

	}

	//给订阅主题发数据
	function publish() {
		var topic = 'SERVER_H5_KEY/' + did;
		var qos = 0;
		var message = '发送内容';
		var retain = true;
		logMessage("INFO", "正在发布消息: [Topic: ", topic, ", Payload: ", message, ", QoS: ", qos, ", Retain: ", retain, "]");
		message = new Paho.Message(message);
		message.destinationName = topic;
		message.qos = Number(qos);
		message.retained = retain;
		client.send(message);
	}

	//订阅
	function subscribe() {
		var topic = 'SERVER_H5_KEY/' + did;
		var qos = 0;
		logMessage("INFO", "订阅: [Topic: ", topic, ", QoS: ", qos, "]");
		client.subscribe(topic, {
			qos: Number(qos)
		});
	}
	//取消订阅
	function unsubscribe() {
		var topic = 'SERVER_H5_KEY/' + did;
		logMessage("INFO", "取消订阅: [Topic: ", topic, "]");
		client.unsubscribe(topic, {
			onSuccess: unsubscribeSuccess,
			onFailure: unsubscribeFailure,
			invocationContext: {
				topic: topic
			}
		});
	}

	//取消订阅成功
	function unsubscribeSuccess(context) {
		logMessage("INFO", "未订阅. [Topic: ", context.invocationContext.topic, "]");
	}
	//取消订阅失败
	function unsubscribeFailure(context) {
		logMessage("ERROR", "取消订阅失败. [Topic: ", context.invocationContext.topic, ", Error: ", context.errorMessage, "]");
	}
	// 以防有人发送HTML
	function safeTagsRegex(str) {
		return str.replace(/&/g, "&amp;").replace(/</g, "&lt;").
		replace(/>/g, "&gt;");
	}

	//制作ID
	function makeid() {
		var text = "";
		var possible = "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz0123456789";

		for (var i = 0; i < 5; i++)
			text += possible.charAt(Math.floor(Math.random() * possible.length));

		return text;
	}
	//消息
	function logMessage(type, ...content) {
		if (type === "INFO") {
			console.info(new Date().toString() + '信息——————————————————————' + content);
		} else if (type === "ERROR") {
			console.error('失败——————————————————————' + content);
		} else {
			console.log(logMessage);
		}
	}
	// 当客户端连接时调用
	function onConnect(context) {
		// 建立连接后，进行订阅并发送消息。
		var connectionString = context.invocationContext.host + ":" + context.invocationContext.port + context.invocationContext
			.path;
		logMessage("INFO", "连接成功 ", "[URI: ", connectionString, ", ID: ", context.invocationContext.clientId, "]");
		console.log("已连接到: " + connectionString + " as " + context.invocationContext.clientId)
		subscribe()
		connected = true;

	}

	function onConnected(reconnect, uri) {
		// 建立连接后，进行订阅并发送消息。
		logMessage("INFO", "订阅现在已连接: [重新连接: ", reconnect, ", URI: ", uri, "]");
		connected = true;
	}

	function onFail(context) {
		logMessage("ERROR", "连接失败. [错误消息: ", context.errorMessage, "]");
		console.log("连接失败: " + context.errorMessage)
		connected = false;
	}
	// 当订阅失去连接时调用
	function onConnectionLost(responseObject) {
		if (responseObject.errorCode !== 0) {
			logMessage("INFO", "连接丢失. [错误消息: ", responseObject.errorMessage, "]");
		}
		connected = false;
	}
	// 当消息到达时呼叫
	function onMessageArrived(message) {
		logMessage("INFO", "收到的消息: [Topic: ", message.destinationName, ", Payload: ", message.payloadString, ", QoS: ",
			message.qos, ", Retained: ", message.retained, ", Duplicate: ", message.duplicate, "]");
		var List = JSON.parse(message.payloadString);
		renderDomMqtt(List)
	}
}
